import sqlite3
from xianbao.core.DatabaseLanguageEnhanced import AdvancedDatabaseLanguageFactory
from typing import Dict, List, Any
from xianbao.core.DatabaseInterface import DatabaseInterface
from xianbao.core.CustomSQLInterface import CustomSQLInterface
from xianbao.core.ast import DMLGenerator


class SQLiteDatabase(DatabaseInterface, CustomSQLInterface):
    """SQLite数据库操作实现"""
    
    def __init__(self, db_name: str = 'rpa_ztdb.db'):
        """初始化SQLite数据库连接
        
        Args:
            db_name: 数据库文件名
        """
        self.db_path = db_name
        self.connection = None
        self.cursor = None
        _, self.dml = AdvancedDatabaseLanguageFactory.create()
        self.dml_generator = DMLGenerator()
    
    def _get_connection(self) -> sqlite3.Connection:
        """获取数据库连接"""
        if not self.connection:
            self.connection = sqlite3.connect(self.db_path, detect_types=sqlite3.PARSE_COLNAMES)
            self.connection.row_factory = sqlite3.Row  # 允许以字典方式访问结果
        return self.connection

    def query(self, **kwargs) -> List[Dict]:
        """执行查询操作

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                fields: ["字段列表"]
                where: {条件表达式}
                order_by: "排序字段"
                limit: 数量
                offset: 偏移量
                joins: [联表配置]
                group_by: [分组字段]
                having: {分组条件}

        Returns:
            查询结果列表
        """
        if 'table' not in kwargs:
            raise ValueError("table参数不能为空")
        
        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST格式参数
            sql, params = self.dml_generator.dict_to_select(kwargs)
            cursor.execute(sql, params)
            rows = cursor.fetchall()
            
            # 将结果转换为字典形式
            columns = [desc[0] for desc in cursor.description]
            return [dict(zip(columns, row)) for row in rows]
    
    def update(self, **kwargs) -> int:
        """执行更新操作

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                set: {要更新的字段和值字典}
                where: {条件表达式}

        Returns:
            受影响的行数
        """
        if 'table' not in kwargs:
            raise ValueError("table参数不能为空")
        
        if 'set' not in kwargs:
            raise ValueError("set参数不能为空")

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST生成SQL和参数
            sql, params = self.dml_generator.dict_to_update(kwargs)
            cursor.execute(sql, params)
            conn.commit()
            return cursor.rowcount
    
    def insert(self, **data) -> Any:
        """执行插入操作

        Args:
            **data: 要插入的字段和值，包含：
                table: "表名"
                其他字段: 值

        Returns:
            最后插入的行ID
        """
        if 'table' not in data:
            raise ValueError("table参数不能为空")

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST格式参数
            sql, params = self.dml_generator.dict_to_insert(data)
            cursor.execute(sql, params)
            conn.commit()
            return cursor.lastrowid
    
    def begin_transaction(self) -> None:
        """开始事务"""
        self._get_connection().execute("BEGIN")
    
    def commit_transaction(self) -> None:
        """提交事务"""
        if self.connection:
            self.connection.commit()
    
    def rollback_transaction(self) -> None:
        """回滚事务"""
        if self.connection:
            self.connection.rollback()
    
    def delete(self, **kwargs) -> int:
        """执行删除操作

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                where: {条件表达式}

        Returns:
            删除的行数
        """
        if 'table' not in kwargs:
            raise ValueError("table参数不能为空")

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST生成SQL和参数
            sql, params = self.dml_generator.dict_to_delete(kwargs)
            cursor.execute(sql, params)
            conn.commit()
            return cursor.rowcount
    
    def count(self, **kwargs) -> int:
        """执行计数操作

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                where: {条件表达式}

        Returns:
            符合条件的记录数
        """
        if 'table' not in kwargs:
            raise ValueError("table参数不能为空")

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST生成SQL
            sql, params = self.dml_generator.dict_to_count(kwargs)
            cursor.execute(sql, params)
            return cursor.fetchone()[0]
    
    def close(self) -> None:
        """关闭数据库连接"""
        if self.connection:
            self.connection.close()
            self.connection = None
            self.cursor = None
    
    def test_connection(self) -> Dict[str, Any]:
        """测试数据库连接
        
        执行完整的数据库连接测试，包括文件访问权限和基本查询测试。
        
        Returns:
            包含连接测试结果的详细信息：
            - success: 是否连接成功
            - message: 测试结果描述
            - response_time: 响应时间（毫秒）
            - file_path: 数据库文件路径
            - file_exists: 数据库文件是否存在
            - file_size: 数据库文件大小（字节）
            - error: 错误信息（如果失败）
        """
        import time
        import os
        
        start_time = time.time()
        result = {
            'success': False,
            'message': '',
            'response_time': 0,
            'file_path': self.db_path,
            'file_exists': False,
            'file_size': 0,
            'error': None
        }
        
        try:
            # 检查数据库文件是否存在
            result['file_exists'] = os.path.exists(self.db_path)
            
            if result['file_exists']:
                result['file_size'] = os.path.getsize(self.db_path)
            
            # 测试数据库连接
            conn = self._get_connection()
            cursor = conn.cursor()
            
            # 测试基本查询
            cursor.execute("SELECT 1 as test")
            test_result = cursor.fetchone()
            
            if test_result and test_result[0] == 1:
                response_time = int((time.time() - start_time) * 1000)
                result['response_time'] = response_time
                result['success'] = True
                
                if result['file_exists']:
                    result['message'] = f'连接成功，数据库文件正常 ({result["file_size"]} 字节)'
                else:
                    result['message'] = '连接成功，已创建新的数据库文件'
            else:
                result['message'] = '连接失败，查询测试未返回预期结果'
                result['error'] = '查询测试失败'
                
        except sqlite3.Error as e:
            result['message'] = '数据库连接失败'
            result['error'] = f"SQLite错误: {str(e)}"
        except Exception as e:
            result['message'] = '未知错误'
            result['error'] = str(e)
            
        return result
    
    def execute_custom_sql(self, sql: str, params: tuple = None) -> List[Dict[str, Any]]:
        """执行自定义SQL查询语句（DML）

        Args:
            sql: 要执行的SQL语句
            params: SQL参数元组，用于参数化查询

        Returns:
            查询结果列表，每个元素是一个字典，键为列名，值为对应的数据
        """
        with self._get_connection() as conn:
            cursor = conn.cursor()
            if params:
                cursor.execute(sql, params)
            else:
                cursor.execute(sql)

            rows = cursor.fetchall()

            # 将结果转换为字典形式
            if cursor.description:  # 检查是否有结果集
                columns = [desc[0] for desc in cursor.description]
                return [dict(zip(columns, row)) for row in rows]
            else:
                return []
    
    def execute_custom_update(self, sql: str, params: tuple = None) -> int:
        """执行自定义更新语句（DML/DDL）

        Args:
            sql: 要执行的SQL语句（UPDATE, INSERT, DELETE, DDL等）
            params: SQL参数元组，用于参数化查询

        Returns:
            受影响的行数（对于DML语句）或0（对于DDL语句）
        """
        with self._get_connection() as conn:
            cursor = conn.cursor()
            try:
                if params:
                    cursor.execute(sql, params)
                else:
                    cursor.execute(sql)

                # 对于DDL语句，rowcount可能为-1，此时返回0
                affected_rows = max(0, cursor.rowcount)

                # 自动提交事务
                conn.commit()
                return affected_rows

            except Exception as e:
                # 发生错误时回滚事务
                conn.rollback()
                raise e
    
    def query_with_pagination(self, **kwargs) -> List[Dict]:
        """分页查询方法

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                fields: ["字段列表"]
                where: {条件表达式}
                order_by: "排序字段"
                limit: 数量
                offset: 偏移量
                ascending: 排序方向（True升序，False降序）

        Returns:
            查询结果列表
        """
        if 'table' not in kwargs:
            raise ValueError("table参数不能为空")

        # 处理排序方向
        ascending = kwargs.pop('ascending', True)
        order_by = kwargs.get('order_by')
        
        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST格式参数
            sql, params = self.dml_generator.dict_to_select(kwargs)
            
            # 处理排序方向
            if order_by and not ascending:
                sql = sql.replace(f"ORDER BY {order_by}", f"ORDER BY {order_by} DESC")

            cursor.execute(sql, params)
            rows = cursor.fetchall()

            # 将结果转换为字典形式
            columns = [desc[0] for desc in cursor.description]
            return [dict(zip(columns, row)) for row in rows]
    
    def query_with_sorting(self, **kwargs) -> List[Dict]:
        """排序查询方法

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                where: {条件表达式}
                order_by: "排序字段"
                ascending: 排序方向（True升序，False降序）

        Returns:
            查询结果列表
        """
        return self.query_with_pagination(**kwargs)
    
    def batch_insert(self, **kwargs) -> List[Any]:
        """批量插入方法

        Args:
            **kwargs: 参数包含：
                table: "表名"
                data_list: [要插入的数据字典列表]

        Returns:
            插入的行ID列表
        """
        # 从kwargs中获取table和data_list参数
        table = kwargs.pop('table', None)
        if not table:
            raise ValueError("table参数不能为空")
        
        data_list = kwargs.pop('data_list', None)
        if not data_list:
            return []
        
        inserted_ids = []
        
        with self._get_connection() as conn:
            cursor = conn.cursor()
            try:
                conn.execute("BEGIN")
                
                for data in data_list:
                    sql, params = self.dml_generator.dict_to_insert(table, data)
                    cursor.execute(sql, params)
                    inserted_ids.append(cursor.lastrowid)
                
                conn.commit()
                return inserted_ids
                
            except Exception as e:
                conn.rollback()
                raise Exception(f"Batch insert failed: {str(e)}")
    
    def batch_update(self, **kwargs) -> int:
        """批量更新方法

        Args:
            **kwargs: 参数包含：
                table: "表名"
                conditions_list: [条件字典列表]
                updates_list: [更新字典列表]

        Returns:
            受影响的行数
        """
        # 从kwargs中获取table参数
        table = kwargs.pop('table', None)
        if not table:
            raise ValueError("table参数不能为空")
        
        conditions_list = kwargs.pop('conditions_list', None)
        updates_list = kwargs.pop('updates_list', None)
        
        if not conditions_list or not updates_list or len(conditions_list) != len(updates_list):
            return 0
        
        total_affected = 0
        
        with self._get_connection() as conn:
            cursor = conn.cursor()
            try:
                conn.execute("BEGIN")
                
                for conditions, updates in zip(conditions_list, updates_list):
                    update_params = {'set': updates}
                    if conditions:
                        update_params['where'] = conditions
                    sql, params = self.dml_generator.dict_to_update(table, update_params)
                    cursor.execute(sql, params)
                    total_affected += cursor.rowcount
                
                conn.commit()
                return total_affected
                
            except Exception as e:
                conn.rollback()
                raise Exception(f"Batch update failed: {str(e)}")
    
    def batch_delete(self, **kwargs) -> int:
        """批量删除方法

        Args:
            **kwargs: 参数包含：
                table: "表名"
                conditions_list: [条件字典列表]

        Returns:
            受影响的行数
        """
        # 从kwargs中获取table参数
        table = kwargs.pop('table', None)
        if not table:
            raise ValueError("table参数不能为空")
        
        conditions_list = kwargs.pop('conditions_list', None)
        
        if not conditions_list:
            return 0
        
        total_affected = 0
        
        with self._get_connection() as conn:
            cursor = conn.cursor()
            try:
                conn.execute("BEGIN")
                
                for conditions in conditions_list:
                    delete_params = {}
                    if conditions:
                        delete_params['where'] = conditions
                    sql, params = self.dml_generator.dict_to_delete(table, delete_params)
                    cursor.execute(sql, params)
                    total_affected += cursor.rowcount
                
                conn.commit()
                return total_affected
                
            except Exception as e:
                conn.rollback()
                raise Exception(f"Batch delete failed: {str(e)}")
    
    def exists(self, **kwargs) -> bool:
        """检查记录是否存在

        Args:
            **kwargs: AST格式参数，支持：
                table: "表名"
                where: {条件表达式}

        Returns:
            记录是否存在
        """
        # 从kwargs中获取table参数
        table = kwargs.pop('table', None)
        if not table:
            raise ValueError("table参数不能为空")

        # 构建查询参数字典
        query_params = {'fields': ["1"]}
        if 'where' in kwargs:
            query_params['where'] = kwargs['where']

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST生成SQL
            sql, params = self.dml_generator.dict_to_select(table, query_params)
            sql = sql.replace("SELECT 1", "SELECT COUNT(*) as count")
            
            cursor.execute(sql, params)
            return cursor.fetchone()[0] > 0
    
    def get_field_values(self, **kwargs) -> List[Any]:
        """获取指定字段的值列表

        Args:
            **kwargs: 参数包含：
                table: "表名"
                field: "字段名"
                where: {条件表达式}
                distinct: 是否去重（默认False）

        Returns:
            字段值列表
        """
        # 从kwargs中获取table和field参数
        table = kwargs.pop('table', None)
        if not table:
            raise ValueError("table参数不能为空")
        
        field = kwargs.pop('field', None)
        if not field:
            raise ValueError("field参数不能为空")

        # 构建查询参数字典
        query_params = {}
        if 'where' in kwargs:
            query_params['where'] = kwargs['where']

        # 处理distinct和字段
        distinct = kwargs.pop('distinct', False)
        query_field = f"DISTINCT {field}" if distinct else field
        query_params['fields'] = [query_field]

        with self._get_connection() as conn:
            cursor = conn.cursor()
            # 使用AST生成SQL
            sql, params = self.dml_generator.dict_to_select(table, query_params)
            
            cursor.execute(sql, params)
            return [row[0] for row in cursor.fetchall()]
            


import json
import tkinter as tk
import threading
import datetime
from tkinter import ttk, filedialog, messagebox
from tkinter.scrolledtext import ScrolledText
from xianbao.core.context import global_context
from xianbao.service.rpa_backfilling_recong import ExcelWriter
from apscheduler.triggers.cron import CronTrigger
from apscheduler.schedulers.background import BackgroundScheduler


class ReconciliationTab(ttk.Frame):
    """
    对账回填功能的主界面类
    
    Args:
        parent: 父窗口组件
    """
    def __init__(self, parent):
        super().__init__(parent)
        self.task_manager = global_context.get('task_manager')
        self.dict_manager = None
        self.is_running = False
        self.scheduler = BackgroundScheduler()
        self.create_widgets()
        self.load_config()
        
    def _init_dict_manager(self):
        """初始化DictManager单例"""
        if self.dict_manager is None:
            self.dict_manager = global_context.get('dict_manager')
        
    def load_config(self):
        """从字典加载保存的配置"""
        self._init_dict_manager()
        
        # 从JSON加载所有配置参数
        config_json = self.dict_manager.get_dict_value("sys_recon", "对账配置")
        if config_json:
            try:
                config_dict = json.loads(config_json)
                self.cron_entry.delete(0, tk.END)
                self.cron_entry.insert(0, config_dict.get("cron_expr", ""))
                
                recon_file = config_dict.get("recon_file", "")
                self.recon_file_label.config(text=recon_file)
                    
                trans_file = config_dict.get("trans_file", "")
                self.trans_file_label.config(text=trans_file)
            except json.JSONDecodeError:
                # 配置解析失败时全部置空
                self.cron_entry.delete(0, tk.END)
                self.recon_file_label.config(text="")
                self.trans_file_label.config(text="")
    
    def create_widgets(self):
        """创建对账回填界面组件"""
        # 第一行：Cron表达式
        ttk.Label(self, text="Cron表达式:").grid(row=0, column=0, padx=5, pady=5, sticky="w")
        self.cron_entry = ttk.Entry(self, width=30)
        self.cron_entry.grid(row=0, column=1, padx=5, pady=5, sticky="we")
        self.cron_entry.insert(0, "0 9 * * *")
        
        # 第二行：对账文件选择
        ttk.Button(self, text="选择对账文件", command=lambda: self.select_file("对账文件")).grid(row=1, column=0, padx=5, pady=5)
        self.recon_file_label = ttk.Label(self, text="未选择文件")
        self.recon_file_label.grid(row=1, column=1, padx=5, pady=5, sticky="w")
        
        # 第三行：流水文件选择
        ttk.Button(self, text="选择流水文件", command=lambda: self.select_file("流水文件")).grid(row=2, column=0, padx=5, pady=5)
        self.trans_file_label = ttk.Label(self, text="未选择文件")
        self.trans_file_label.grid(row=2, column=1, padx=5, pady=5, sticky="w")
        
        # 第四行：按钮区域
        btn_frame = ttk.Frame(self)
        btn_frame.grid(row=3, column=0, columnspan=2, padx=5, pady=5, sticky="we")
        
        # 开始对账按钮(仅验证Cron)
        self.sync_btn = ttk.Button(btn_frame, text="开始对账(Cron)", command=self.start_reconciliation_by_cron)
        self.sync_btn.pack(side="left", expand=True, fill="x")
        
        # 直接开始对账按钮
        self.direct_sync_btn = ttk.Button(btn_frame, text="直接开始对账", command=self.start_reconciliation_directly)
        self.direct_sync_btn.pack(side="left", expand=True, fill="x")
        
        # 保存配置按钮
        self.save_btn = ttk.Button(btn_frame, text="保存配置", command=self.save_config)
        self.save_btn.pack(side="left", expand=True, fill="x")
        
        # 第五行：状态标签
        self.status_label = ttk.Label(self, text="运行状态: 空闲")
        self.status_label.grid(row=4, column=0, columnspan=2, padx=5, pady=5, sticky="w")
        
        # 定时任务状态标签
        self.scheduler_status_label = ttk.Label(self, text="定时任务: 已关闭")
        self.scheduler_status_label.grid(row=4, column=1, columnspan=1, padx=5, pady=5, sticky="e")
        
        # 第六行：日志框
        self.log_text = ScrolledText(self, height=15, state="disabled")
        self.log_text.grid(row=5, column=0, columnspan=2, padx=5, pady=5, sticky="nsew")
        
        self.grid_columnconfigure(1, weight=1)
        self.grid_rowconfigure(5, weight=1)
    
    def select_file(self, file_type):
        """选择对账或流水文件"""
        file_path = self._get_file_path()
        if file_path:
            self._update_file_label(file_type, file_path)
            
    def _get_file_path(self):
        """获取文件路径"""
        return filedialog.askopenfilename(
            filetypes=[("Excel文件", ".xlsx .xlsm")]
        )
        
    def _update_file_label(self, file_type, file_path):
        """更新文件标签"""
        if file_type == "对账文件":
            self.recon_file_label.config(text=file_path)
        else:
            self.trans_file_label.config(text=file_path)
    
    def start_reconciliation_by_cron(self):
        """通过Cron表达式验证后执行对账"""
        if self.is_running:
            return
            
        # 如果定时任务已开启，则关闭它
        if hasattr(self, 'scheduler') and self.scheduler.running:
            self.scheduler.remove_job('rpa_reconciliation_job')
            self.scheduler.shutdown()
            self.scheduler_status_label.config(text="定时任务: 已关闭")
            # messagebox.showinfo("成功", "定时任务已关闭")
            return
            
        cron_expr = self.cron_entry.get()
        try:
            # croniter.croniter(cron_expr)
            
            # 检查是否已有定时任务，有则先移除
            if hasattr(self, 'scheduler') and self.scheduler.get_jobs():
                self.scheduler.remove_all_jobs()
                
            params = self.parse_cron_expr(cron_expr)

            # 创建定时任务
            self.scheduler.add_job(
                self._execute_reconciliation,
                trigger=CronTrigger(
                    second=params['second'],
                    minute=params['minute'],
                    hour=params['hour'],
                    day=params['day'],
                    month=params['month'],
                    day_of_week=params['day_of_week']
                ),
                id='rpa_reconciliation_job'
            )
            self.scheduler.start()
            self.scheduler_status_label.config(text="定时任务: 已开启")
            
            messagebox.showinfo("成功", f"已创建定时任务: {cron_expr}")
        except Exception as e:
            messagebox.showerror("错误", f"无效的Cron表达式: {e}")
    

    def parse_cron_expr(self, cron_expr):
        """
        解析Cron表达式为APScheduler参数
        """
        parts = cron_expr.split()
        
        # 确保有6个部分
        if len(parts) != 6:
            raise ValueError("Cron表达式必须包含6个部分")
            
        # 处理特殊字符*，转换为None
        params = {} 
        params['second'] = parts[0]
        params['minute'] = parts[1]
        params['hour'] = parts[2]
        params['day'] = parts[3]
        params['month'] = parts[4]
        params['day_of_week'] = parts[5]
        return params


    def save_config(self):
        """保存配置到字典"""
        config_data = self._get_config_data()
        self._save_to_dict(config_data)
        messagebox.showinfo("成功", "配置已保存")
        
    def _get_config_data(self):
        """获取配置数据"""
        return {
            "cron_expr": self.cron_entry.get(),
            "recon_file": self.recon_file_label.cget("text"),
            "trans_file": self.trans_file_label.cget("text")
        }
        
    def _save_to_dict(self, config_data):
        """保存配置到字典"""
        dict_manager = global_context.get('dict_manager')
        
        # 添加字典类型(如果不存在)
        if not dict_manager.get_dict_type_by_code("sys_recon"):
            dict_manager.add_dict_type("对账配置", "sys_recon")
            
        # 保存所有配置为字典并持久化为JSON
        import json
        config_dict = {
            "cron_expr": config_data["cron_expr"],
            "recon_file": config_data["recon_file"],
            "trans_file": config_data["trans_file"]
        }
        
        # 检查字典项是否存在，不存在则新增
        if not dict_manager.get_dict_value("sys_recon", "对账配置"):
            dict_manager.add_dict_item("sys_recon", "对账配置", json.dumps(config_dict, ensure_ascii=False))
        else:
            dict_manager.update_dict_item("sys_recon", "对账配置", json.dumps(config_dict, ensure_ascii=False))
    
    def start_reconciliation_directly(self):
        """直接开始对账任务"""
        if self.is_running:
            messagebox.showwarning("提示", "当前已有对账任务运行中")
            return
            
        # 启动对账
        self._execute_reconciliation()
        
    def _execute_reconciliation(self):
        """执行对账任务"""
        # 更新UI状态
        self._update_ui_state(running=True)
        
        # 启动后台线程
        thread = threading.Thread(
            target=self.run_reconciliation, 
            args=(self.recon_file_label.cget("text"), self.trans_file_label.cget("text"))
        )
        thread.daemon = True
        thread.start()
        
    def _update_ui_state(self, running):
        """更新UI状态"""
        self.is_running = running
        state = "disabled" if running else "!disabled"
        self.sync_btn.config(state=state)
        self.direct_sync_btn.config(state=state)
        self.save_btn.config(state=state)
        self.status_label.config(text=f"运行状态: {'运行中' if running else '空闲'}")
        state_text = "运行中" if running else "空闲"
        button_state = "disabled" if running else "normal"
        
        self.status_label.config(text=f"运行状态: {state_text}")
        self.scheduler_status_label.config(text=f"定时任务: {'已开启' if self.scheduler.running else '已关闭'}")
        self.sync_btn.config(state=button_state)
        self.direct_sync_btn.config(state=button_state)
        self.save_btn.config(state=button_state)
        
        if running:
            self.log_text.config(state="normal")
            self.log_text.delete(1.0, tk.END)
            self.log_text.config(state="disabled")
    
    def run_reconciliation(self, recon_path, trans_path):
        """执行对账任务"""
        handler = ExcelWriter(recon_path, trans_path)
        page = 1
        per_page = 1000
        
        while True:
            tasks = self.rpa_backfilling(page, per_page)
            if not tasks:
                break
                
            for task in tasks:
                try:
                    f = handler.write_to_excel(task)
                    if f:
                        self.update_log(f"处理任务 {task['id']} 成功")
                    else:
                        self.update_log(f"处理任务 {task['id']} 失败")
                except Exception as e:
                    self.update_log(f"处理任务 {task['id']} 失败: {str(e)}")
            
            page += 1
        
        handler.close()
        
        # 恢复界面状态
        self._update_ui_state(running=False)
        self.update_log("所有对账任务处理完成")
    
    def rpa_backfilling(self, page=1, per_page=1000):
        """分页获取任务列表"""
        offset = (page - 1) * per_page
        sql = f"""
            SELECT * FROM tasks 
            ORDER BY created_time DESC 
            LIMIT {per_page} OFFSET {offset}
        """
        tasks = self.task_manager.execute_custom_sql(sql)

        # 反序列化meta字段
        for task in tasks:
            if 'meta' in task and task['meta']:
                task['meta'] = json.loads(task['meta'])

        return tasks
    
    def update_log(self, message):
        """更新日志"""
        timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        log_message = f"[{timestamp}] {message}\n"
        
        self.log_text.config(state="normal")
        self.log_text.insert("end", log_message)
        self.log_text.see("end")
        self.log_text.config(state="disabled")


import tkinter as tk
from tkinter import ttk

class StatusVisualization(ttk.Frame):
    """16位状态可视化组件"""
    
    STATUS_BITS = {
        15: ("预留位p", "p", "N/A"),
        14: ("预留位o", "o", "N/A"),
        13: ("预留位n", "n", "N/A"),
        12: ("预留位m", "m", "N/A"),
        11: ("参考号识别失败", "l", "N/A"),
        10: ("回填失败", "k", "已完成"),
        9: ("回填成功", "j", "待处理"),
        8: ("送货单识别", "i", "已完成"),
        7: ("发票识别", "h", "待处理"),
        6: ("宜家小票识别", "g", "已完成"),
        5: ("银联小票识别", "f", "待处理"),
        4: ("商品图片入库", "e", "已完成"),
        3: ("送货单入库", "d", "待处理"),
        2: ("发票入库", "c", "已完成"),
        1: ("宜家小票入库", "b", "待处理"),
        0: ("银联小票入库", "a", "已完成")
    }
    
    def __init__(self, master=None, **kwargs):
        super().__init__(master, **kwargs)
        self.configure(padding=5)
        self.create_widgets()
    
    def create_widgets(self):
        """创建状态灯网格"""
        self.lights = []
        
        for bit in range(16):
            label, char, _ = self.STATUS_BITS[bit]
            
            # 创建状态灯
            light = tk.Canvas(self, width=20, height=20, bg="gray", highlightthickness=0)
            light.grid(row=bit//8, column=(bit%8)*2, padx=2, pady=2)
            
            # 创建标签
            tk.Label(self, text=f"{char}: {label}").grid(row=bit//8, column=(bit%8)*2+1, sticky=tk.W)
            
            # 添加工具提示
            self.create_tooltip(light, label)
            
            self.lights.append(light)
    
    def create_tooltip(self, widget, text):
        """为组件添加工具提示"""
        tooltip = tk.Toplevel(self)
        tooltip.withdraw()
        tooltip.overrideredirect(True)
        
        label = tk.Label(tooltip, text=text, bg="lightyellow", 
                       relief="solid", borderwidth=1)
        label.pack()
        
        def enter(event):
            bbox = widget.bbox("all")
            if bbox is not None:
                x, y, _, _ = bbox
                x += widget.winfo_rootx() + 25
                y += widget.winfo_rooty() + 25
                tooltip.geometry(f"+{x}+{y}")
                tooltip.deiconify()
            
        def leave(event):
            tooltip.withdraw()
            
        widget.bind("<Enter>", enter)
        widget.bind("<Leave>", leave)
    
    def update_status(self, status):
        """更新状态显示"""
        for bit in range(1, 17):
            color = "green" if status & (1 << (16 - bit)) else "red"
            self.lights[bit - 1].configure(bg=color)




from .Task import TaskManager
from .TaskApi import TaskApi
from .DictData import DictManager
from .DictData import DictApi
from .BlockDeque import SQLiteProcessQueue, BusyError
from .context import global_context
from .DatabaseLanguageEnhanced import AdvancedDatabaseLanguageFactory
from .ProcessQueueInterface import ProcessQueueInterface, ProcessQueueBase
from .HttpSQLiteProcessQueue import HttpSQLiteProcessQueue
from .QueueFactory import QueueFactory, QueueConfigBuilder, QueueManager

__all__ = [
    'TaskManager', 'TaskApi', 'DictManager', 'DictApi', 
    'SQLiteProcessQueue', 'BusyError', 'global_context', 
    'AdvancedDatabaseLanguageFactory', 'ProcessQueueInterface', 
    'ProcessQueueBase', 'HttpSQLiteProcessQueue',
    'QueueFactory', 'QueueConfigBuilder', 'QueueManager'
]




import json
from datetime import datetime
from typing import Dict, List, Optional, Tuple, Any
from xianbao.core.DatabaseInterface import DatabaseInterface


class TaskApi:
    """RPA任务数据库API - 基于DatabaseLanguageEnhanced优化实现
    
    注意：所有where条件语法遵循AST模块的最佳实践，使用明确的运算符格式
    例如：{"id": {"=": 123}} 而不是简写的 {"id": 123}
    """

    # 允许的排序字段白名单
    ALLOWED_ORDER_FIELDS = {"id", "status", "created_time", "updated_time", "business_type", "sort_order"}

    def __init__(self, table_name: str, db_language: tuple, db_interface: DatabaseInterface = None):
        """初始化任务管理API
        
        Args:
            table_name: 任务表名，默认为"tasks"
            db_language: 数据库语言元组，包含DDL和DML语句
            db_interface: 数据库接口实现，如果提供则直接使用
        """
        if db_interface is None:
            raise ValueError("db_interface must be provided")
        
        self.TABLE_NAME = table_name
        self.TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
        self.db_interface = db_interface
        self.ddl, self.dml = db_language
        self.initialize_db()

    def _validate_order_field(self, field: str) -> str:
        """验证排序字段是否在白名单中，防止SQL注入"""
        if field not in self.ALLOWED_ORDER_FIELDS:
            raise ValueError(f"Invalid order field: {field}")
        return field

    def initialize_db(self) -> None:
        """初始化数据库和表结构"""
        try:
            # 检查表是否已存在
            existing_tables = self.db_interface.query(
                table="sqlite_master",
                where={
                    "AND": [
                        {"type": {"=": "table"}},
                        {"name": {"=": self.TABLE_NAME}}
                    ]
                }
            )
            
            if not existing_tables:
                # 表不存在，创建表和索引
                self.db_interface.begin_transaction()
                try:
                    # 创建表
                    columns = {
                        "id": "INTEGER PRIMARY KEY AUTOINCREMENT",
                        "status": "TEXT NOT NULL",
                        "meta": "TEXT",
                        "created_time": "TEXT NOT NULL",
                        "updated_time": "TEXT NOT NULL",
                        "business_type": "TEXT",
                        "log": "TEXT",
                        "sort_order": "INTEGER"
                    }
                    
                    # 使用自定义SQL创建表和索引
                    create_table_sql = self.ddl.create_table(self.TABLE_NAME, columns)
                    self.db_interface.execute_custom_update(create_table_sql)
                    
                    # 创建索引
                    index_sql1 = self.ddl.create_index(self.TABLE_NAME, "status", "idx_status")
                    index_sql2 = self.ddl.create_index(self.TABLE_NAME, "business_type", "idx_business_type")
                    index_sql3 = self.ddl.create_index(self.TABLE_NAME, "created_time", "idx_created_time")
                    index_sql4 = self.ddl.create_index(self.TABLE_NAME, "sort_order", "idx_sort_order")
                    
                    self.db_interface.execute_custom_update(index_sql1)
                    self.db_interface.execute_custom_update(index_sql2)
                    self.db_interface.execute_custom_update(index_sql3)
                    self.db_interface.execute_custom_update(index_sql4)
                    
                    self.db_interface.commit_transaction()
                except Exception as e:
                    self.db_interface.rollback_transaction()
                    raise e
        except Exception as e:
            raise Exception(f"Failed to initialize database: {str(e)}")

    def add_task(self, meta: Dict, business_type: str, status: int = 0) -> int:
        """添加新任务"""
        now = datetime.now().strftime(self.TIME_FORMAT)
        meta_json = json.dumps(meta)

        try:
            self.db_interface.begin_transaction()
            task_id = self.db_interface.insert(
                table=self.TABLE_NAME,
                data={
                    "status": str(status),
                    "meta": meta_json,
                    "created_time": now,
                    "updated_time": now,
                    "business_type": business_type
                }
            )
            self.db_interface.commit_transaction()
            return task_id
        except Exception as e:
            self.db_interface.rollback_transaction()
            raise Exception(f"Failed to add task: {str(e)}")

    def update_status(self, task_id: int, new_status: int) -> bool:
        """更新任务状态"""
        now = datetime.now().strftime(self.TIME_FORMAT)

        try:
            self.db_interface.begin_transaction()
            affected = self.db_interface.update(
                table=self.TABLE_NAME,
                set={
                    "status": str(new_status),
                    "updated_time": now
                },
                where={"id": {"=": task_id}}
            )
            self.db_interface.commit_transaction()
            return affected > 0
        except Exception as e:
            self.db_interface.rollback_transaction()
            raise Exception(f"Failed to update task status: {str(e)}")

    def update_status_and_meta(self, task_id: int, new_status: int, new_meta: Dict) -> bool:
        """同时更新任务状态和meta字段"""
        now = datetime.now().strftime(self.TIME_FORMAT)
        meta_json = json.dumps(new_meta)

        try:
            self.db_interface.begin_transaction()
            affected = self.db_interface.update(
                table=self.TABLE_NAME,
                set={
                    "status": str(new_status),
                    "meta": meta_json,
                    "updated_time": now
                },
                where={"id": {"=": task_id}}
            )
            self.db_interface.commit_transaction()
            return affected > 0
        except Exception as e:
            self.db_interface.rollback_transaction()
            raise Exception(f"Failed to update task status and meta: {str(e)}")

    def add_task_with_sort(self, meta: Dict, status: int = 0, business_type: str = None) -> int:
        """添加任务并自动设置排序值"""
        try:
            self.db_interface.begin_transaction()
            
            # 获取当前最大排序值
            max_sort_result = self.db_interface.execute_custom_sql(
                sql="SELECT MAX(sort_order) as max_sort FROM " + self.TABLE_NAME
            )
            max_sort = max_sort_result[0]["max_sort"] if max_sort_result and max_sort_result[0]["max_sort"] is not None else 0
            
            # 插入新任务
            task_id = self.db_interface.insert(
                table=self.TABLE_NAME,
                data={
                    "status": str(status),
                    "meta": json.dumps(meta),
                    "created_time": datetime.now().strftime(self.TIME_FORMAT),
                    "updated_time": datetime.now().strftime(self.TIME_FORMAT),
                    "business_type": business_type,
                    "sort_order": max_sort + 1
                }
            )
            
            self.db_interface.commit_transaction()
            return task_id
            
        except Exception as e:
            self.db_interface.rollback_transaction()
            raise Exception(f"Failed to add task with sort: {str(e)}")

    def update_log(self, task_id: int, log: str) -> bool:
        """更新任务日志"""
        try:
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                where={"id": {"=": task_id}}
            )
            
            if not result:
                return False
                
            task = result[0]
            meta = json.loads(task["meta"])
            if "logs" not in meta:
                meta["logs"] = []
            meta["logs"].append({
                "time": datetime.now().strftime(self.TIME_FORMAT),
                "content": log
            })
            
            affected = self.db_interface.update(
                table=self.TABLE_NAME,
                set={
                    "meta": json.dumps(meta),
                    "updated_time": datetime.now().strftime(self.TIME_FORMAT)
                },
                where={"id": {"=": task_id}}
            )
            
            return affected > 0
            
        except Exception as e:
            raise Exception(f"Failed to update task log: {str(e)}")

    def update_sort_order(self, task_id: int, sort_order: int) -> bool:
        """更新任务的排序值"""
        now = datetime.now().strftime(self.TIME_FORMAT)

        try:
            affected = self.db_interface.update(
                table=self.TABLE_NAME,
                set={
                    "sort_order": sort_order,
                    "updated_time": now
                },
                where={"id": {"=": task_id}}
            )
            return affected > 0
        except Exception as e:
            raise Exception(f"Failed to update task sort order: {str(e)}")

    def get_tasks_by_status(self, status: int) -> List[Dict]:
        """根据状态获取任务列表"""
        try:
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                where={"status": {"=": str(status)}}
            )
            
            return [self._row_to_dict(row) for row in result]
            
        except Exception as e:
            raise Exception(f"Failed to get tasks by status: {str(e)}")

    def list_tasks_by_status(
            self,
            status: int,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """根据状态分页查询任务"""
        order_by = self._validate_order_field(order_by)
        offset = (page - 1) * per_page
        
        try:
            # 使用增强的DML进行分页查询
            order_direction = "ASC" if ascending else "DESC"
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                where={"status": {"=": str(status)}},
                order_by=f"{order_by} {order_direction}",
                limit=per_page,
                offset=offset
            )
            
            return [self._row_to_dict(row) for row in result]
        except Exception as e:
            raise Exception(f"Failed to list tasks by status: {str(e)}")

    def count_by_status(self, status: int) -> int:
        """获取指定状态的记录数量"""
        try:
            result = self.db_interface.execute_custom_sql(
                sql="SELECT COUNT(*) as count FROM " + self.TABLE_NAME + " WHERE status = ?",
                params=[str(status)]
            )
            return result[0]["count"] if result else 0
        except Exception as e:
            raise Exception(f"Failed to count tasks by status: {str(e)}")

    def get_task_by_id(self, task_id: int) -> Optional[Dict]:
        """根据ID获取任务"""
        try:
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                where={"id": {"=": task_id}}
            )
            
            if result:
                task = result[0]
                return {
                    "id": task["id"],
                    "status": task["status"],
                    "meta": json.loads(task["meta"]),
                    "created_time": task["created_time"],
                    "updated_time": task["updated_time"],
                    "business_type": task["business_type"],
                    "sort_order": task["sort_order"]
                }
            return None
            
        except Exception as e:
            raise Exception(f"Failed to get task by ID: {str(e)}")

    def batch_update_status(self, task_ids: List[int], new_status: int) -> int:
        """批量更新任务状态"""
        if not task_ids:
            return 0

        now = datetime.now().strftime(self.TIME_FORMAT)

        try:
            affected = 0
            self.db_interface.begin_transaction()
            
            for task_id in task_ids:
                affected += self.db_interface.update(
                table=self.TABLE_NAME,
                set={
                    "status": str(new_status),
                    "updated_time": now
                },
                where={"id": {"=": task_id}}
            )
            
            self.db_interface.commit_transaction()
            return affected
        except Exception as e:
            self.db_interface.rollback_transaction()
            raise Exception(f"Failed to batch update task status: {str(e)}")

    def delete_task(self, task_id: int) -> bool:
        """删除任务"""
        try:
            affected = self.db_interface.delete(
                table=self.TABLE_NAME,
                where={"id": {"=": task_id}}
            )
            return affected > 0
        except Exception as e:
            raise Exception(f"Failed to delete task: {str(e)}")

    def health_check(self) -> Tuple[bool, str]:
        """检查数据库连接状态"""
        try:
            self.db_interface.execute_custom_sql(
                sql=f"SELECT COUNT(*) FROM {self.TABLE_NAME}",
                params=[]
            )
            return True, "Database is healthy"
        except Exception as e:
            return False, f"Database error: {str(e)}"

    def list_all_tasks(
            self,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """分页查询所有任务"""
        order_by = self._validate_order_field(order_by)
        offset = (page - 1) * per_page
        
        try:
            # 使用增强的DML进行分页查询
            order_direction = "ASC" if ascending else "DESC"
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                order_by=f"{order_by} {order_direction}",
                limit=per_page,
                offset=offset
            )
            
            return [self._row_to_dict(row) for row in result]
        except Exception as e:
            raise Exception(f"Failed to list all tasks: {str(e)}")

    def count_all_tasks(self) -> int:
        """获取整个表格的总记录数"""
        try:
            result = self.db_interface.execute_custom_sql(
                sql=f"SELECT COUNT(*) as count FROM {self.TABLE_NAME}",
                params=[]
            )
            return result[0]["count"] if result else 0
        except Exception as e:
            raise Exception(f"Failed to count all tasks: {str(e)}")

    def advanced_search(
            self,
            status: Optional[int] = None,
            created_start: Optional[str] = None,
            created_end: Optional[str] = None,
            business_type: Optional[str] = None,
            meta_contains: Optional[Dict] = None,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> Tuple[List[Dict], int]:
        """高级分页搜索"""
        order_by = self._validate_order_field(order_by)
        try:
            conditions = {}
            
            # 状态条件
            if status is not None:
                conditions["status"] = {"=": str(status)}
            
            # 业务类型条件
            if business_type is not None:
                conditions["business_type"] = {"=": business_type}
            
            # 创建时间条件
            if created_start or created_end:
                time_conditions = []
                if created_start:
                    time_conditions.append({"created_time": {">=": created_start + " 00:00:00"}})
                if created_end:
                    time_conditions.append({"created_time": {"<=": created_end + " 23:59:59"}})
                
                if len(time_conditions) > 1:
                    conditions["AND"] = time_conditions
                elif len(time_conditions) == 1:
                    conditions.update(time_conditions[0])
            
            # 获取总数
            total_tasks = self.db_interface.query(
                table=self.TABLE_NAME,
                where=conditions
            )
            
            # 应用meta_contains过滤
            filtered_tasks = []
            for task in total_tasks:
                task_dict = self._row_to_dict(task)
                
                # 应用meta_contains过滤
                if meta_contains:
                    match = True
                    for key, value in meta_contains.items():
                        if task_dict["meta"].get(key) != value:
                            match = False
                            break
                    if match:
                        filtered_tasks.append(task_dict)
                else:
                    filtered_tasks.append(task_dict)
            
            # 排序
            reverse = not ascending
            filtered_tasks.sort(key=lambda x: x[order_by], reverse=reverse)
            
            # 分页处理
            total_count = len(filtered_tasks)
            offset = (page - 1) * per_page
            result = filtered_tasks[offset:offset + per_page]
            
            return result, total_count
        except Exception as e:
            raise Exception(f"Failed to perform advanced search: {str(e)}")

    def parse_datetime(self, dt_str: str) -> datetime:
        """将数据库时间字符串解析为datetime对象"""
        return datetime.strptime(dt_str, self.TIME_FORMAT)

    def format_datetime(self, dt: datetime) -> str:
        """将datetime对象格式化为数据库字符串"""
        return dt.strftime(self.TIME_FORMAT)

    def list_tasks_created_between(
            self,
            start_time: str,
            end_time: str,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """获取指定时间段内创建的任务"""
        order_by = self._validate_order_field(order_by)
        offset = (page - 1) * per_page
        
        try:
            order_direction = "ASC" if ascending else "DESC"
            result = self.db_interface.query(
                table=self.TABLE_NAME,
                where={
                    "AND": [
                        {"created_time": {">=": start_time}},
                        {"created_time": {"<=": end_time}}
                    ]
                },
                order_by=f"{order_by} {order_direction}",
                limit=per_page,
                offset=offset
            )
            
            return [self._row_to_dict(row) for row in result]
        except Exception as e:
            raise Exception(f"Failed to list tasks created between: {str(e)}")

    def list_tasks_by_business_type(self, business_type: str, 
                                 order_by: str = "created_time", limit: int = None, 
                                 offset: int = None) -> List[Dict]:
        """按业务类型查询任务"""
        order_by = self._validate_order_field(order_by)
        
        try:
            conditions = {"business_type": {"=": business_type}}
            
            tasks = self.db_interface.query(
                table=self.TABLE_NAME,
                where=conditions,
                order_by=f"{order_by} ASC",
                limit=limit,
                offset=offset
            )
            
            return [self._row_to_dict(task) for task in tasks]
        except Exception as e:
            raise Exception(f"Failed to list tasks by business type: {str(e)}")

    def count_tasks_by_business_type(self, business_type: str) -> int:
        """统计指定业务类型的任务数量"""
        try:
            result = self.db_interface.execute_custom_sql(
                sql="SELECT COUNT(*) as count FROM " + self.TABLE_NAME + " WHERE business_type = ?",
                params=[business_type]
            )
            return result[0]["count"] if result else 0
        except Exception as e:
            raise Exception(f"Failed to count tasks by business type: {str(e)}")

    def _row_to_dict(self, row) -> Dict:
        """将数据库行转换为字典格式"""
        return {
            "id": row["id"],
            "status": int(row["status"]),
            "meta": json.loads(row["meta"]),
            "created_time": row["created_time"],
            "updated_time": row["updated_time"],
            "business_type": row["business_type"],
            "sort_order": row["sort_order"]
        }

    def close(self) -> None:
        """关闭数据库连接"""
        self.db_interface.close()
    
    def is_custom_sql_supported(self) -> bool:
        """检查当前数据库实现是否支持自定义SQL执行
        
        Returns:
            如果支持自定义SQL执行返回True，否则返回False
        """
        from xianbao.core.CustomSQLInterface import CustomSQLInterface
        return isinstance(self.db_interface, CustomSQLInterface)
    
    def execute_custom_sql(self, sql: str, params: tuple = None) -> List[Dict[str, Any]]:
        """执行自定义SQL查询语句（DML）
        
        Args:
            sql: 要执行的SQL语句
            params: SQL参数元组，用于参数化查询
            
        Returns:
            查询结果列表，每个元素是一个字典，键为列名，值为对应的数据
            
        Raises:
            NotImplementedError: 当数据库实现不支持自定义SQL执行时
            Exception: 当SQL执行失败时
        """
        if not self.is_custom_sql_supported():
            raise NotImplementedError("当前数据库实现不支持自定义SQL执行")
        
        return self.db_interface.execute_custom_sql(sql, params)
    
    def execute_custom_update(self, sql: str, params: tuple = None) -> int:
        """执行自定义更新语句（DML/DDL）
        
        Args:
            sql: 要执行的SQL语句（UPDATE, INSERT, DELETE, DDL等）
            params: SQL参数元组，用于参数化查询
            
        Returns:
            受影响的行数（对于DML语句）或0（对于DDL语句）
            
        Raises:
            NotImplementedError: 当数据库实现不支持自定义SQL执行时
            Exception: 当SQL执行失败时
        """
        if not self.is_custom_sql_supported():
            raise NotImplementedError("当前数据库实现不支持自定义SQL执行")
        
        return self.db_interface.execute_custom_update(sql, params)
        


from setuptools import setup, find_packages

setup(
    name='xianb',
    version='2.0.5',
    author='jxjiang',
    author_email='723137901@qq.com',
    description='很大的修改，改到颠了，不想干了',
    python_requires='>=3.8',
    install_requires=[],
    packages=find_packages()
)



import numpy as np
import pandas as pd
import datetime
from decimal import Decimal
import base64
import json

def json_serializer(obj):
    """将常见非标准Python类型转换为JSON可序列化格式"""
    if isinstance(obj, (np.integer, np.int_)):
        return int(obj)
    elif isinstance(obj, (np.floating, np.float_)):
        return float(obj)
    elif isinstance(obj, (np.ndarray, pd.Series)):
        return obj.tolist()  # 转换数组为列表
    elif isinstance(obj, pd.Timestamp):
        return obj.isoformat()  # 转换时间戳为ISO字符串
    elif isinstance(obj, datetime.datetime):
        return obj.isoformat()
    elif isinstance(obj, datetime.date):
        return obj.isoformat()
    elif isinstance(obj, Decimal):
        return float(obj)  # 或 str(obj) 根据需求
    elif isinstance(obj, bytes):
        # 处理bytes类型数据
        return base64.b64encode(obj).decode('utf-8')
    elif hasattr(obj, 'to_dict'):
        # 支持Pandas DataFrame等可转换为字典的对象
        return obj.to_dict()
    elif hasattr(obj, 'to_json'):
        # 支持直接转换为JSON的对象
        return json.loads(obj.to_json())
    elif isinstance(obj, pd.DataFrame):
        # DataFrame特殊处理
        return obj.to_dict(orient='records')
    
    # 对于其他类型，可以添加更多处理，或者抛出异常
    raise TypeError(f"类型 {type(obj)} 不可JSON序列化")

def convert_payload(payload):
    """递归转换payload中的所有元素为JSON兼容类型"""
    if isinstance(payload, (list, tuple)):
        return [convert_payload(item) for item in payload]
    elif isinstance(payload, dict):
        return {k: convert_payload(v) for k, v in payload.items()}
    else:
        try:
            # 尝试直接序列化Python原生类型
            json.dumps(payload)
            return payload
        except TypeError:
            # 使用自定义序列化器处理复杂类型
            return json_serializer(payload)


import tkinter as tk
import threading
import datetime
import json
from tkinter import ttk, filedialog, messagebox
from tkinter.scrolledtext import ScrolledText
from xianbao.core.context import global_context
from xianbao.service.rpa_backfilling_qk import ExcelWriter
from apscheduler.triggers.cron import CronTrigger
from apscheduler.schedulers.background import BackgroundScheduler


class BackfillTab(ttk.Frame):
    """
    数据回填功能的主界面类，继承自ttk.Frame
    
    Args:
        parent: 父窗口组件
    """
    def __init__(self, parent):
        super().__init__(parent)
        self.task_manager = global_context.get('task_manager')  # 任务管理器实例
        if self.task_manager is None:
            raise Exception("任务管理器未初始化")
        self.is_running = False  # 任务运行状态标志
        self.create_widgets()  # 创建界面组件
        self.load_params()  # 加载保存的参数
        
    def load_params(self):
        """
        从字典中加载保存的参数
        """
        dict_manager = global_context['dict_manager']
        
        # 获取保存的参数
        params = dict_manager.get_dict_items("backfill_params")
        if params:
            try:
                param_dict = json.loads(params[0]["value"])
                self.cron_entry.delete(0, tk.END)
                self.cron_entry.insert(0, param_dict.get("cron_expr", "0 9 * * *"))
                self.file_label.config(text=param_dict.get("excel_path", "未选择文件"))
            except Exception as e:
                print(f"加载参数失败: {e}")
        
    def create_widgets(self):
        """
        创建并布局所有界面组件
        """
        # Cron表达式输入
        ttk.Label(self, text="Cron表达式:").grid(row=0, column=0, padx=5, pady=5, sticky="w")
        self.cron_entry = ttk.Entry(self, width=30)
        self.cron_entry.grid(row=0, column=1, padx=5, pady=5, sticky="we")
        self.cron_entry.insert(0, "0 9 * * *")  # 默认每天9点执行
        
        # Excel文件选择
        ttk.Button(self, text="选择RPA回填文件", command=self.select_file).grid(row=1, column=0, padx=5, pady=5)
        self.file_label = ttk.Label(self, text="未选择文件")
        self.file_label.grid(row=1, column=1, padx=5, pady=5, sticky="w")
        
        # 按钮容器框架
        self.btn_frame = ttk.Frame(self)
        self.btn_frame.grid(row=2, column=0, columnspan=2, padx=5, pady=3, sticky="we")
        
        # 保存配置按钮
        self.save_btn = ttk.Button(self.btn_frame, text="保存配置", command=self.save_params, width=12)
        self.save_btn.pack(side="left", expand=False, padx=3)
        
        # 同步按钮
        self.sync_btn = ttk.Button(self.btn_frame, text="同步数据", command=self.start_backfill, width=12)
        self.sync_btn.pack(side="left", expand=False, padx=3)
        
        # 定时任务生效按钮
        self.schedule_btn = ttk.Button(self.btn_frame, text="定时任务", command=self.toggle_schedule, width=12)
        self.schedule_btn.pack(side="left", expand=False, padx=3)
        
        # 定时任务状态标识
        self.schedule_status = ttk.Label(self, text="定时任务状态: 未生效", padding=(5, 5))
        self.schedule_status.grid(row=3, column=1, columnspan=5, padx=5, pady=5, sticky="we")
        
        # 添加空行分隔
        ttk.Label(self).grid(row=3)
        
        # 状态标签
        self.status_label = ttk.Label(self, text="状态: 空闲")
        self.status_label.grid(row=3, column=0, columnspan=2, padx=5, pady=5, sticky="w")
        
        # 日志框
        self.log_text = ScrolledText(self, height=15, state="disabled")
        self.log_text.grid(row=4, column=0, columnspan=2, padx=5, pady=5, sticky="nsew")
        
        self.grid_columnconfigure(1, weight=1)
        self.grid_rowconfigure(4, weight=1)
    
    def select_file(self):
        """
        打开文件选择对话框，选择Excel数据源文件
        """
        # 确保在主线程中处理文件对话框
        self.after(100, lambda: self._process_file_selection())
        
    def _process_file_selection(self):
        """
        实际处理文件选择的内部方法
        """
        try:
            file_path = filedialog.askopenfilename(
                filetypes=[("Excel文件", ".xlsx .xlsm")],
                title="选择Excel文件"
            )
            if file_path:
                self.file_label.config(text=file_path)  # 更新文件路径显示
        except Exception as e:
            messagebox.showerror("错误", f"文件选择失败: {str(e)}")
    
    def save_params(self):
        """
        保存当前配置参数到字典
        """
        # 验证Cron表达式
        cron_expr = self.cron_entry.get()
        # 验证Excel文件路径
        excel_path = self.file_label.cget("text")
        if not excel_path or excel_path == "未选择文件":
            messagebox.showerror("错误", "请先选择Excel文件")
            return
            
        # 保存参数到字典
        dict_manager = global_context['dict_manager']
        
        # 检查是否已存在相同配置
        # 确保字典类型存在
        if not dict_manager.get_dict_type_by_code("backfill_params"):
            dict_manager.add_dict_type("回填参数", "backfill_params", "RPA回填任务参数")
            
        existing_data = dict_manager.get_dict_items("backfill_params")
        params = {
            "cron_expr": cron_expr,
            "excel_path": excel_path
        }
        
        if existing_data:
            # 更新现有配置
            dict_manager.update_dict_item("backfill_params", existing_data[0]["key"], json.dumps(params))
        else:
            # 创建新配置
            dict_manager.add_dict_item("backfill_params", "default", json.dumps(params))
        
        messagebox.showinfo("成功", "参数已保存")
        
    def toggle_schedule(self):
        """
        切换定时任务状态
        """
        if not hasattr(self, 'scheduler'):
            self.scheduler = BackgroundScheduler()
            self.scheduler.start()
            self.is_scheduled = False
        
        if self.is_scheduled:
            # 取消定时任务
            self.scheduler.remove_job('rpa_backfill_job')
            self.is_scheduled = False
            self.schedule_status.config(text="定时任务: 未生效")
            return
        
        # 设置定时任务
        cron_expr = self.cron_entry.get()
        try:
            # 验证Cron表达式
            # croniter.croniter(cron_expr)
            print(f"尝试添加定时任务，Cron表达式: {cron_expr}")
            params = self.parse_cron_expr(cron_expr)
            # 添加定时任务
            self.scheduler.add_job(
                self.start_backfill,
                trigger=CronTrigger(
                    second=params['second'],
                    minute=params['minute'],
                    hour=params['hour'],
                    day=params['day'],
                    month=params['month'],
                    day_of_week=params['day_of_week']
                ),
                id='rpa_backfill_job'
            )
            print("定时任务添加成功")
            self.is_scheduled = True
            self.schedule_status.config(text="定时任务: 已生效")
        except Exception as e:
            messagebox.showerror("错误", f"定时任务设置失败: {str(e)}")
            print(f"定时任务设置失败: {str(e)}")
            
    def parse_cron_expr(self, cron_expr):
        """
        解析Cron表达式为APScheduler参数
        """
        parts = cron_expr.split()
        
        # 确保有6个部分
        if len(parts) != 6:
            raise ValueError("Cron表达式必须包含6个部分")
            
        # 处理特殊字符*，转换为None
        params = {} 
        params['second'] = parts[0]
        params['minute'] = parts[1]
        params['hour'] = parts[2]
        params['day'] = parts[3]
        params['month'] = parts[4]
        params['day_of_week'] = parts[5]
        return params
    
    def start_backfill(self):
        """
        启动数据回填任务
        
        验证输入参数后，创建后台线程执行回填任务
        """
        if self.is_running:
            return  # 防止重复启动
            
        # 验证Excel文件路径
        excel_path = self.file_label.cget("text")
        if not excel_path or excel_path == "未选择文件":
            messagebox.showerror("错误", "请先选择Excel文件")
            return
        
            
        # 更新状态并禁用按钮
        self.is_running = True
        self.status_label.config(text="状态: 运行中")
        self.sync_btn.config(state="disabled")
        
        # 创建并启动后台线程
        thread = threading.Thread(target=self.run_backfill, args=(excel_path,))
        thread.daemon = True  # 设置为守护线程
        thread.start()
    
    def run_backfill(self, excel_path):
        """
        执行数据回填任务的核心方法
        
        Args:
            excel_path: Excel数据源文件路径
        """
        excel_writer = ExcelWriter(xlsx_path=excel_path, db_path=self.db_path)  # 创建处理器实例
        page = 1
        per_page = 1000  # 每页处理1000条任务
        
        # 分页处理所有任务
        while True:
            tasks = self.rpa_backfilling(page, per_page)  # 获取任务列表
            if not tasks:
                break  # 没有更多任务时退出循环
                
            # 处理当前页的所有任务
            for task in tasks:
                try:
                    excel_writer.write_to_excel(task)  # 执行单个任务
                    self.update_log(f"处理任务成功，任务ID: {str(task['id'])}")
                except Exception as e:
                    self.update_log(f"处理任务失败: {str(e)}，任务ID: {str(task['id'])}")
            
            page += 1  # 处理下一页

        excel_writer.close()

        # 任务完成后恢复界面状态
        self.is_running = False
        self.status_label.config(text="状态: 空闲")
        self.sync_btn.config(state="normal")
        self.update_log("所有任务处理完成")

    def rpa_backfilling(self, page=1, per_page=1000):
        """
        分页获取任务列表
        
        Args:
            page: 当前页码
            per_page: 每页记录数
            
        Returns:
            当前页的任务列表
        """
        offset = (page - 1) * per_page
        query = "SELECT * FROM tasks ORDER BY created_time DESC LIMIT ? OFFSET ?"
        params = (per_page, offset)
        
        tasks = self.task_manager.execute_custom_sql(query, params)
        
        # 反序列化meta字段
        for task in tasks:
            if 'meta' in task and task['meta']:
                task['meta'] = json.loads(task['meta'])
                
        return tasks
    
    def update_log(self, message):
        """
        更新日志显示区域
        
        Args:
            message: 要显示的日志消息
        """
        timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        log_message = f"[{timestamp}] {message}\n"
        
        # 更新日志文本框内容
        self.log_text.config(state="normal")
        self.log_text.insert("end", log_message)
        self.log_text.see("end")  # 自动滚动到底部
        self.log_text.config(state="disabled")
        


from typing import Dict, Any, Optional, List
from xianbao.core.ProcessQueueInterface import ProcessQueueInterface, ProcessQueueBase
from xianbao.core.BlockDeque import SQLiteProcessQueue
from xianbao.core.HttpSQLiteProcessQueue import HttpSQLiteProcessQueue


class QueueFactory:
    """
    队列工厂类，负责创建和管理各种队列实现
    
    提供统一的接口来创建不同类型的进程安全队列，
    支持SQLite本地队列和HTTP远程队列等多种实现。
    """
    
    # 支持的队列类型
    QUEUE_TYPES = {
        'sqlite': SQLiteProcessQueue,
        'http': HttpSQLiteProcessQueue,
        'http_lock': HttpSQLiteProcessQueue,
    }
    
    @classmethod
    def create_queue(cls, 
                    queue_type: str = 'sqlite',
                    **kwargs) -> ProcessQueueInterface:
        """
        创建指定类型的队列实例
        
        Args:
            queue_type: 队列类型，支持 'sqlite', 'http', 'http_lock'
            **kwargs: 传递给具体队列实现的参数
            
        Returns:
            队列实例
            
        Raises:
            ValueError: 当指定的队列类型不支持时
        """
        if queue_type not in cls.QUEUE_TYPES:
            raise ValueError(
                f"不支持的队列类型: {queue_type}. "
                f"支持的类型: {list(cls.QUEUE_TYPES.keys())}"
            )
        
        queue_class = cls.QUEUE_TYPES[queue_type]
        
        # 根据队列类型处理特定参数
        if queue_type == 'sqlite':
            return cls._create_sqlite_queue(**kwargs)
        elif queue_type == 'http':
            kwargs.pop('use_lock', None)  # 移除可能冲突的参数
            return cls._create_http_queue(use_lock=False, **kwargs)
        elif queue_type == 'http_lock':
            kwargs.pop('use_lock', None)  # 移除可能冲突的参数
            return cls._create_http_queue(use_lock=True, **kwargs)
    
    @classmethod
    def _create_sqlite_queue(cls, **kwargs) -> SQLiteProcessQueue:
        """创建SQLite本地队列"""
        # 设置SQLite队列的默认参数
        sqlite_defaults = {
            'db_path': 'rpa_ztdb.db',
            'table_name': 'queue',
            'max_size': 10000,
            'timeout': 30.0,
            'lock_path': 'rpa_queue',
            'journal_mode': 'WAL',
            'busy_timeout': 5000,
            'poll_interval': 0.1,
            'max_poll_interval': 1.0
        }
        
        # 合并用户参数和默认参数
        config = {**sqlite_defaults, **kwargs}
        return SQLiteProcessQueue(**config)
    
    @classmethod
    def _create_http_queue(cls, use_lock: bool = False, **kwargs) -> HttpSQLiteProcessQueue:
        """创建HTTP远程队列"""
        # 设置HTTP队列的默认参数
        http_defaults = {
            'base_url': 'http://localhost:5000',
            'table_name': 'http_queue',
            'max_size': 10000,
            'timeout': 30.0
        }
        
        # 合并用户参数和默认参数
        config = {**http_defaults, **kwargs}
        config['use_lock'] = use_lock
        return HttpSQLiteProcessQueue(**config)
    
    @classmethod
    def get_supported_types(cls) -> List[str]:
        """获取支持的队列类型列表"""
        return list(cls.QUEUE_TYPES.keys())
    
    @classmethod
    def get_default_config(cls, queue_type: str) -> Dict[str, Any]:
        """
        获取指定队列类型的默认配置
        
        Args:
            queue_type: 队列类型
            
        Returns:
            默认配置字典
            
        Raises:
            ValueError: 当指定的队列类型不支持时
        """
        if queue_type not in cls.QUEUE_TYPES:
            raise ValueError(f"不支持的队列类型: {queue_type}")
        
        if queue_type == 'sqlite':
            return {
                'db_path': 'rpa_ztdb.db',
                'table_name': 'queue',
                'max_size': 10000,
                'timeout': 30.0,
                'lock_path': 'rpa_queue',
                'journal_mode': 'WAL',
                'busy_timeout': 5000,
                'poll_interval': 0.1,
                'max_poll_interval': 1.0
            }
        elif queue_type in ['http', 'http_lock']:
            return {
                'base_url': 'http://localhost:5000',
                'table_name': 'http_queue',
                'max_size': 10000,
                'timeout': 30.0,
                'use_lock': queue_type == 'http_lock'
            }


class QueueConfigBuilder:
    """
    队列配置构建器，用于简化队列配置的创建
    """
    
    def __init__(self, queue_type: str):
        """
        初始化配置构建器
        
        Args:
            queue_type: 队列类型
        """
        self.queue_type = queue_type
        self.config = {}
    
    def with_db_path(self, db_path: str) -> 'QueueConfigBuilder':
        """设置数据库路径（仅SQLite队列）"""
        self.config['db_path'] = db_path
        return self
    
    def with_base_url(self, base_url: str) -> 'QueueConfigBuilder':
        """设置HTTP服务基础URL（仅HTTP队列）"""
        self.config['base_url'] = base_url
        return self
    
    def with_table_name(self, table_name: str) -> 'QueueConfigBuilder':
        """设置队列表名"""
        self.config['table_name'] = table_name
        return self
    
    def with_max_size(self, max_size: int) -> 'QueueConfigBuilder':
        """设置队列最大容量"""
        self.config['max_size'] = max_size
        return self
    
    def with_timeout(self, timeout: float) -> 'QueueConfigBuilder':
        """设置操作超时时间"""
        self.config['timeout'] = timeout
        return self
    
    def with_use_lock(self, use_lock: bool) -> 'QueueConfigBuilder':
        """设置是否使用锁模式（仅HTTP队列）"""
        self.config['use_lock'] = use_lock
        return self
    
    def build(self) -> ProcessQueueInterface:
        """根据配置创建队列实例"""
        return QueueFactory.create_queue(self.queue_type, **self.config)
    
    def get_config(self) -> Dict[str, Any]:
        """获取当前配置字典"""
        return self.config.copy()


class QueueManager:
    """
    队列管理器，提供队列的集中管理和监控
    """
    
    def __init__(self):
        """初始化队列管理器"""
        self._queues: Dict[str, ProcessQueueInterface] = {}
    
    def create_and_register(self, 
                         name: str,
                         queue_type: str,
                         **kwargs) -> ProcessQueueInterface:
        """
        创建并注册队列实例
        
        Args:
            name: 队列名称（用于后续管理）
            queue_type: 队列类型
            **kwargs: 队列参数
            
        Returns:
            创建的队列实例
        """
        if name in self._queues:
            raise ValueError(f"队列名称已存在: {name}")
        
        queue = QueueFactory.create_queue(queue_type, **kwargs)
        self._queues[name] = queue
        return queue
    
    def get_queue(self, name: str) -> Optional[ProcessQueueInterface]:
        """获取已注册的队列实例"""
        return self._queues.get(name)
    
    def remove_queue(self, name: str) -> bool:
        """移除并关闭队列实例"""
        if name in self._queues:
            queue = self._queues.pop(name)
            queue.close()
            return True
        return False
    
    def list_queues(self) -> List[str]:
        """获取所有已注册的队列名称"""
        return list(self._queues.keys())
    
    def close_all(self):
        """关闭所有已注册的队列"""
        for name, queue in self._queues.items():
            queue.close()
        self._queues.clear()
    
    def get_queue_info(self, name: str) -> Optional[Dict[str, Any]]:
        """获取队列的基本信息"""
        queue = self.get_queue(name)
        if queue is None:
            return None
        
        return {
            'name': name,
            'type': type(queue).__name__,
            'size': queue.qsize(),
            'empty': queue.empty(),
            'full': queue.full()
        }
    
    def get_all_info(self) -> Dict[str, Dict[str, Any]]:
        """获取所有队列的信息"""
        return {
            name: self.get_queue_info(name)
            for name in self.list_queues()
        }



import json
import os
from datetime import datetime
from textwrap import indent
from typing import Dict, Any, Optional
from xianbao.core.ast import QueryBuilder
from xianbao.core.Task import TaskManager
from xianbao.core.DatabaseFactory import DatabaseFactory
from xianbao.core.DatabaseLanguageEnhanced import AdvancedDatabaseLanguageFactory
from xianbao.core.QueueFactory import QueueFactory
from xianbao.core.ProcessQueueInterface import ProcessQueueInterface


def initialize_service(host: str='http://localhost:5000') -> Dict[str, Any]:
    """
    初始化服务组件，使用工厂模式创建数据库和队列实例
    
    Args:
        _db_path: 数据库路径
        db_type: 数据库类型 ('sqlite' 或 'http_sqlite')
        queue_type: 队列类型 ('sqlite', 'http', 'http_lock')
    
    Returns:
        包含服务组件的字典
    """

    # 创建数据库实例
    db_config = { 'base_url': host, 'use_lock': False }

    database = DatabaseFactory.create_database('http_sqlite', db_config)
    
    # 创建数据库语言元组 (DDL, DML)
    db_language = AdvancedDatabaseLanguageFactory.create('http_sqlite')
    
    # 创建任务管理器
    task_manager = TaskManager(
        db_interface=database,
        db_language=db_language,
        table_name='tasks'
    )
    
    # 创建队列实例    
    queue_config = { 'base_url': host, 'use_lock': True, 'table_name': 'entering_queue' }
        
    
    queue = QueueFactory.create_queue('http_lock', **queue_config)
    
    return {
        # 'db_path': db_config['db_path'],
        # 'lock_path': queue_config.get('lock_path', ''),
        'task_manager': task_manager,
        'queue': queue
        # 'database': database,
        # 'db_language': db_language
    }


def _scan_task_production(task_manager, queue):
    """
    生产待录入表单的任务。
    状态：全部图片入库成功、识别成功、未走过录入流程
    使用AST模块优化查询条件
    :return: None
    """
    
    # 使用QueryBuilder构建复杂查询
    builder = QueryBuilder('tasks')
    
    # 入库成功 + 识别成功 (status & 0b1011110110000000 = 0b1011110110000000)
    # 未录入状态 (status & 0b0000000001100000 = 0)
    # 参考号识别错误 (status & 0b0000000000010000 = 0)
    
    # 使用位运算条件
    conditions = {
        'AND': [
            {'status': {'&': int('1011110110000000', 2)}},
            {'status': {'&': int('0000000001100000', 2), '=': 0}},
            {'status': {'&': int('0000000000010000', 2), '=': 0}}
        ]
    }
    
    sql, params = builder.select('*').where(**conditions).to_sql()
    
    # 使用TaskManager的底层数据库接口执行查询
    tasks = task_manager.execute_custom_sql(sql, params)
    
    for task in tasks:
        task['meta'] = json.loads(task['meta'])
        queue.put(json.dumps(task, ensure_ascii=False, indent=2))


def _scan_task_get(queue):
    """
    获取待识别任务，并返回消费。
    :return:
    """
    return json.loads(queue.get(indent=2))


def _update_success_task(task_id: int, task_manager):
    """
    交易信息录入成功。
    使用AST模块优化状态更新
    :param task_id: 
    :return: 
    """
    from xianbao.core.ast import update
    
    # 计算要清除的标志位
    clear_flags = int('1000000', 2) | int('10000', 2)
    
    # 使用AST模块构建更新语句
    sql, params = update(**{
        'table': 'tasks',
        'set': {'status': {'&~': clear_flags}},  # 使用位清除操作
        'where': {'id': task_id}
    })
    
    # 执行更新
    task_manager.execute_custom_update(sql, params)


def _update_fail_task(task_id: int, error_msg: str, task_manager):
    """
    交易信息录入失败。
    使用AST模块优化状态更新和日志记录
    :param task_id: 
    :param error_msg: 
    :return: 
    """
    from xianbao.core.ast import update
    
    # 设置失败标志位
    fail_flag = int('100000', 2)
    
    # 获取当前时间
    current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    
    # 使用AST模块构建更新语句
    sql, params = update(**{
        'table': 'tasks',
        'set': {
            'status': {'|': fail_flag},  # 设置失败标志
            'log': {'||': f'[{current_time}] {error_msg}'},  # 追加日志
            'updated_time': current_time
        },
        'where': {'id': task_id}
    })
    
    # 执行更新
    task_manager.execute_custom_update(sql, params)


def _update_invoice_meta(task_id: int, u_meta: dict, task_manager):
    """
    完善发票信息
    使用AST模块优化元数据更新
    :param task_id 任务Id
    :param u_meta 待修改的发票信息
    :return
    """
    from xianbao.core.ast import select, update
    
    # 首先获取当前任务的元数据
    sql, params = select(**{
        'table': 'tasks',
        'fields': ['meta'],
        'where': {'id': task_id}
    })
    
    result = task_manager.execute_custom_sql(sql, params)
    if not result:
        raise ValueError(f"任务 {task_id} 不存在")
    
    current_meta = json.loads(result[0]['meta'])
    
    # 更新发票信息
    if 'invoice_info' not in current_meta:
        current_meta['invoice_info'] = {}
    
    current_meta['invoice_info'].update({
        '发票金额': u_meta['发票金额'],
        '销售企业名称': u_meta['销售企业名称'],
        '统一社会信用代码': u_meta['统一社会信用代码']
    })
    
    # 使用AST模块构建更新语句
    sql, params = update(**{
        'table': 'tasks',
        'set': {
            'meta': json.dumps(current_meta, ensure_ascii=False),
            'updated_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        },
        'where': {'id': task_id}
    })
    
    # 执行更新
    task_manager.execute_custom_update(sql, params)


def fetch_transdate(meta: dict):
    """
    从meta中获取银联小票信息。
    :param meta:
    :return:
    """
    ums_receipt = meta['ums_receipt_info']
    dt_obj = datetime.strptime(ums_receipt['日期时间'], "%Y-%m-%d %H:%M:%S")
    return dt_obj.strftime("%Y/%m/%d")


def fetch_work_folder(entering_task_meta: dict, work_folder: str):
    """
    返回客户的交易信息目录，目录里边包含待回填的所有交易进行。
    :param entering_task_meta: 待回填的消费meta
    :param work_folder: 工作空间
    :return:
    """
    return os.path.join(work_folder, entering_task_meta['relative_path'].strip('\\').strip('/'))


def fetch_area_by_delivery_info(meta: dict):
    """
    从送货单信息（delivery_info）中获取所在区域
    :param meta:
    :return: { 市, 区 }
    """
    area = meta['delivery_info']['地区']
    idx = area.find('市')
    city = area[:idx + 1]
    district = area[idx + 1:]
    return {
        'city': city,
        'district': district
    }


def queue_size(queue: ProcessQueueInterface) -> int:
    """
    获取队列大小
    :param queue: 队列实例
    :return: 队列中的任务数量
    """
    return queue.qsize()


# 向后兼容的别名
__init__ = initialize_service




from xianbao.core.TaskApi import TaskApi
from datetime import datetime
from typing import Dict, List, Optional, Tuple
from xianbao.core.DatabaseInterface import DatabaseInterface


class TaskManager:
    """任务管理器 - 基于TaskApi实现"""

    def __init__(self, db_interface: DatabaseInterface, db_language: tuple, table_name: str = "tasks") -> None:
        """初始化任务管理器"""
        self.task_api = TaskApi(table_name=table_name, db_language=db_language, db_interface=db_interface)
        self.TIME_FORMAT = "%Y-%m-%d %H:%M:%S"

    def close(self) -> None:
        """关闭数据库连接"""
        self.task_api.close()



    def add_task(self, meta: Dict, business_type: str, status: int = 0) -> int:
        """添加新任务"""
        return self.task_api.add_task(meta, business_type, status)

    def update_status(self, task_id: int, new_status: int) -> bool:
        """更新任务状态"""
        return self.task_api.update_status(task_id, new_status)

    def update_status_and_meta(self, task_id: int, new_status: int, new_meta: Dict) -> bool:
        """同时更新任务状态和meta字段"""
        return self.task_api.update_status_and_meta(task_id, new_status, new_meta)

    def add_task_with_sort(self, meta: Dict, status: int = 0, business_type: str = None) -> int:
        """添加任务并自动设置排序值"""
        return self.task_api.add_task_with_sort(meta, status, business_type)

    def update_log(self, task_id: int, log: str) -> bool:
        """更新任务日志"""
        return self.task_api.update_log(task_id, log)

    def update_sort_order(self, task_id: int, sort_order: int) -> bool:
        """更新任务的排序值"""
        return self.task_api.update_sort_order(task_id, sort_order)

    def list_tasks_by_status(
            self,
            status: int,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """根据状态分页查询任务"""
        return self.task_api.list_tasks_by_status(status, page, per_page, order_by, ascending)

    def count_by_status(self, status: int) -> int:
        """获取指定状态的记录数量"""
        return self.task_api.count_by_status(status)

    def get_task(self, task_id: int) -> Optional[Dict]:
        """根据ID获取单个任务详情"""
        return self.task_api.get_task(task_id)

    def batch_update_status(self, task_ids: List[int], new_status: int) -> int:
        """批量更新任务状态"""
        return self.task_api.batch_update_status(task_ids, new_status)

    def delete_task(self, task_id: int) -> bool:
        """删除指定任务"""
        return self.task_api.delete_task(task_id)

    def health_check(self) -> Tuple[bool, str]:
        """检查数据库连接状态"""
        return self.task_api.health_check()

    def list_all_tasks(
            self,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """分页查询所有任务"""
        return self.task_api.list_all_tasks(page, per_page, order_by, ascending)

    def count_all_tasks(self) -> int:
        """获取整个表格的总记录数"""
        return self.task_api.count_all_tasks()

    def advanced_search(
            self,
            status: Optional[int] = None,
            created_start: Optional[str] = None,
            created_end: Optional[str] = None,
            meta_contains: Optional[Dict] = None,
            page: int = 1,
            per_page: int = 10
    ) -> Tuple[List[Dict], int]:
        """高级分页搜索"""
        return self.task_api.advanced_search(status, created_start, created_end, meta_contains, page, per_page)

    def parse_datetime(self, dt_str: str) -> datetime:
        """解析时间字符串为datetime对象"""
        return datetime.strptime(dt_str, self.TIME_FORMAT)

    def format_datetime(self, dt: datetime) -> str:
        """格式化datetime对象为字符串"""
        return dt.strftime(self.TIME_FORMAT)

    def list_tasks_created_between(self, start_time: datetime, end_time: datetime, 
                                order_by: str = "created_time", limit: int = None, 
                                offset: int = None) -> List[Dict]:
        """查询时间段内创建的任务"""
        return self.task_api.list_tasks_created_between(start_time, end_time, order_by, limit, offset)

    def list_tasks_by_business_type(self, business_type: str, 
                                 order_by: str = "created_time", limit: int = None, 
                                 offset: int = None) -> List[Dict]:
        """按业务类型查询任务"""
        return self.task_api.list_tasks_by_business_type(business_type, order_by, limit, offset)

    def count_tasks_by_business_type(self, business_type: str) -> int:
        """统计业务类型任务数量"""
        return self.task_api.count_tasks_by_business_type(business_type)

    def execute_custom_sql(self, sql: str, params: tuple = ()) -> List[Dict]:
        """执行自定义SQL查询（注意SQL注入风险）"""
        return self.task_api.execute_custom_sql(sql, params)

    def execute_custom_update(self, sql: str, params: tuple = ()) -> int:
        """执行自定义SQL更新（注意SQL注入风险）"""
        return self.task_api.execute_custom_update(sql, params)



"""
全局上下文管理器
提供应用程序范围内的共享变量池
"""

class GlobalContext:
    """全局上下文单例类"""
    _instance = None
    _context = {}
    
    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance
    
    @classmethod
    def get_instance(cls):
        """获取全局上下文实例"""
        return cls()
    
    def set(self, key, value):
        """设置全局变量"""
        self._context[key] = value
    
    def get(self, key, default=None):
        """获取全局变量"""
        return self._context.get(key, default)
    
    def update(self, **kwargs):
        """批量更新全局变量"""
        self._context.update(kwargs)
    
    def remove(self, key):
        """移除全局变量"""
        if key in self._context:
            del self._context[key]
    
    def clear(self):
        """清空所有全局变量"""
        self._context.clear()
    
    def keys(self):
        """获取所有键"""
        return list(self._context.keys())
    
    def __getitem__(self, key):
        """支持字典式访问"""
        return self._context[key]
    
    def __setitem__(self, key, value):
        """支持字典式设置"""
        self._context[key] = value
    
    def __contains__(self, key):
        """支持in操作符"""
        return key in self._context

# 创建全局上下文实例
global_context = GlobalContext.get_instance()



from filelock import FileLock, Timeout as FileLockTimeoutError
import sqlite3
import os
import time
from contextlib import contextmanager
from queue import Empty, Full
import logging
from typing import Optional
from xianbao.core.ProcessQueueInterface import ProcessQueueBase


class SQLiteProcessQueue(ProcessQueueBase):
    """
    多进程安全的SQLite阻塞队列

    特性：
    - 进程安全的数据库连接管理
    - 基于文件锁的进程间协调
    - 可选的队列大小限制
    - 高效的数据获取策略
    - 指数退避的等待机制
    - 数据库维护功能

    Note: 每个进程应创建自己的队列实例，指向同一个数据库文件
    """

    def __init__(self, db_path='rpa_ztdb.db', table_name='queue',
                 max_size=10000, timeout=30.0,
                 lock_path='rpa_queue',
                 journal_mode='WAL', busy_timeout=5000,
                 poll_interval=0.1, max_poll_interval=1.0):
        """
        初始化多进程队列

        :param db_path: SQLite数据库文件路径
        :param table_name: 队列表名
        :param max_size: 队列最大容量(可选)
        :param timeout: 默认操作超时时间(秒)
        :param journal_mode: SQLite日志模式(WAL/DELETE等)
        :param busy_timeout: 数据库忙等待超时(毫秒)
        :param poll_interval: 队列空时初始轮询间隔(秒)
        :param max_poll_interval: 最大轮询间隔(秒)
        :param lock_path: 锁文件
        """
        super().__init__(max_size=max_size, timeout=timeout)
        
        self.lock_path = lock_path
        self.db_path = os.path.abspath(db_path)
        self.table_name = table_name
        self.poll_interval = poll_interval
        self.max_poll_interval = max_poll_interval
        self.journal_mode = journal_mode
        self.busy_timeout = busy_timeout

        # 使用 filelock 管理文件锁
        self.lock = FileLock(self.lock_path + ".lock", timeout=timeout)

        # 为每个进程创建独立的数据库连接
        self.conn = None

        # 初始化数据库结构
        self._init_db()

    @contextmanager
    def _db_connection(self):
        """进程安全的数据库连接上下文管理"""
        try:
            if self.conn is None:
                # 创建数据库目录(如果需要)
                os.makedirs(os.path.dirname(self.db_path), exist_ok=True)

                # 创建新连接
                self.conn = sqlite3.connect(
                    self.db_path,
                    timeout=self.busy_timeout / 1000,
                    isolation_level=None  # 使用自动提交模式
                )

                # 优化设置
                self.conn.execute(f'PRAGMA journal_mode={self.journal_mode}')
                self.conn.execute('PRAGMA synchronous=NORMAL')
            yield self.conn
        except sqlite3.OperationalError as e:
            if "database is locked" in str(e):
                raise BusyError("Database is locked") from e
            raise
        except sqlite3.Error as e:
            logging.error(f"Database error: {e}")
            raise
        finally:
            # 不关闭连接，保持打开状态供后续使用
            pass

    def _init_db(self):
        """初始化数据库结构"""
        with self._db_connection() as conn:
            conn.execute(f'''
                CREATE TABLE IF NOT EXISTS {self.table_name} (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    data BLOB NOT NULL,
                    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
                );
            ''')
            # 加速消费查询
            conn.execute(f'CREATE INDEX IF NOT EXISTS idx_order ON {self.table_name}(id ASC);')
            # 检查点清理
            conn.execute('PRAGMA wal_checkpoint(TRUNCATE);')

    def put(self, item: bytes, block=True, timeout=None):
        if not isinstance(item, bytes):
            raise TypeError("Only bytes are supported")

        timeout = timeout or self.timeout
        start_time = time.monotonic()
        poll_interval = self.poll_interval

        while True:
            try:
                with self.lock:
                    current_size = self.qsize()

                    if self.max_size is not None and current_size >= self.max_size:
                        if not block:
                            raise Full("Queue full")

                        elapsed = time.monotonic() - start_time
                        if elapsed >= timeout:
                            raise Full("Queue full")

                        time.sleep(poll_interval)
                        poll_interval = min(poll_interval * 2, self.max_poll_interval)
                        continue

                    with self._db_connection() as conn:
                        conn.execute(
                            f'INSERT INTO {self.table_name} (data) VALUES (?)',
                            (item,)
                        )
                    return

            except FileLockTimeoutError:
                raise TimeoutError("Failed to acquire file lock") from None

    def get(self, block=True, timeout=None) -> bytes:
        timeout = timeout or self.timeout
        start_time = time.monotonic()
        poll_interval = self.poll_interval

        while True:
            try:
                with self.lock:
                    with self._db_connection() as conn:
                        cursor = conn.cursor()
                        cursor.execute(
                            f'DELETE FROM {self.table_name} '
                            'WHERE id = (SELECT id FROM {0} ORDER BY id ASC LIMIT 1) '
                            'RETURNING data'.format(self.table_name)
                        )
                        result = cursor.fetchone()
                        if result:
                            return result[0]

                    if not block:
                        raise Empty("Queue empty")

                    elapsed = time.monotonic() - start_time
                    if elapsed >= timeout:
                        raise Empty("Queue empty")

                    time.sleep(poll_interval)
                    poll_interval = min(poll_interval * 2, self.max_poll_interval)

            except FileLockTimeoutError:
                raise TimeoutError("Failed to acquire file lock") from None

    def qsize(self) -> int:
        """返回队列中的项目数"""
        with self._db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(f'SELECT COUNT(*) FROM {self.table_name}')
            return cursor.fetchone()[0]



    def clear(self) -> None:
        """清空队列"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                conn.execute(f'DELETE FROM {self.table_name}')
        finally:
            self._file_unlock()

    def vacuum(self):
        """回收数据库空间"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                conn.execute('VACUUM')
        finally:
            self._file_unlock()

    def maintenance(self):
        """执行数据库维护任务"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                # WAL检查点
                conn.execute('PRAGMA wal_checkpoint(TRUNCATE);')

                # 重新构建索引
                conn.execute(f'REINDEX {self.table_name}')
        finally:
            self._file_unlock()

    def close(self) -> None:
        """关闭数据库连接"""
        if self.conn:
            try:
                self.conn.close()
            except sqlite3.Error:
                pass
            finally:
                self.conn = None


class BusyError(Exception):
    """数据库忙异常"""
    pass



